"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.refreshRequest = exports.refresh = exports.refreshSIE = exports.refreshSWR = exports.refreshWithStrategy = exports.staleIfError = exports.staleWhileRevalidate = exports.isRefreshing = exports.fromRemoteData = void 0;
var remote_data_ts_1 = require("@devexperts/remote-data-ts");
var function_1 = require("fp-ts/function");
/**
 * Create a {@link RefreshableRemoteData} value from a {@link RemoteData} value
 */
var fromRemoteData = function (current) { return ({
    current: current,
    request: remote_data_ts_1.initial,
}); };
exports.fromRemoteData = fromRemoteData;
/**
 * If a request is pending, the {@link RefreshableRemoteData} is in a "refreshing" state and no
 * further network requests should be sent.
 */
var isRefreshing = function (_a) {
    var request = _a.request;
    return remote_data_ts_1.isPending(request);
};
exports.isRefreshing = isRefreshing;
/**
 * Given a {@link RefreshableRemoteData}, the stale-while-revalidate refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - keep returning the current value until it is replaced by another success or failure value
 * - keep returning a failure value only as long as it is not being refreshed, in which case
 *   it will fall back to a pending value
 *
 * @see {@link staleIfError}
 */
var staleWhileRevalidate = function (current) {
    return remote_data_ts_1.fold(function () { return ({ current: current, request: remote_data_ts_1.initial }); }, function () { return ({ current: remote_data_ts_1.isInitial(current) ? remote_data_ts_1.pending : current, request: remote_data_ts_1.pending }); }, function (e) { return ({ current: remote_data_ts_1.failure(e), request: remote_data_ts_1.initial }); }, function (x) { return ({ current: remote_data_ts_1.success(x), request: remote_data_ts_1.initial }); });
};
exports.staleWhileRevalidate = staleWhileRevalidate;
/**
 * Given a {@link RefreshableRemoteData}, the stale-if-error refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - return a failure only if no data has been previously fetched; newer failures replace older ones
 * - keep returning the cached success value indefinitely unless it is replaced by another success value
 *
 * @see {@link staleWhileRevalidate}
 */
var staleIfError = function (current) {
    return remote_data_ts_1.fold(function () { return ({ current: current, request: remote_data_ts_1.initial }); }, function () { return ({
        current: remote_data_ts_1.isInitial(current) || remote_data_ts_1.isFailure(current) ? remote_data_ts_1.pending : current,
        request: remote_data_ts_1.pending,
    }); }, function (e) { return ({ current: remote_data_ts_1.isSuccess(current) ? current : remote_data_ts_1.failure(e), request: remote_data_ts_1.initial }); }, function (x) { return ({ current: remote_data_ts_1.success(x), request: remote_data_ts_1.initial }); });
};
exports.staleIfError = staleIfError;
/**
 * Build a function to update a {@link RefreshableRemoteData} with a new {@link RemoteData} value
 * using the given strategy.
 */
var refreshWithStrategy = function (strategy) {
    if (strategy === void 0) { strategy = exports.staleWhileRevalidate; }
    return function (rrd) { return strategy(rrd.current); };
};
exports.refreshWithStrategy = refreshWithStrategy;
/**
 * Transition a {@link RefreshableRemoteData} value to the next state using the
 * {@link staleWhileRevalidate} strategy.
 */
exports.refreshSWR = exports.refreshWithStrategy(exports.staleWhileRevalidate);
/**
 * Transition a {@link RefreshableRemoteData} value to the next state using the
 * {@link staleIfError} strategy.
 */
exports.refreshSIE = exports.refreshWithStrategy(exports.staleIfError);
/**
 * Transition a {@link RefreshableRemoteData} value to the next state.
 * @see {@link refreshSWR}
 */
exports.refresh = exports.refreshSWR;
/**
 * Transition a {@link RefreshableRemoteData} value to the next state using a network request. The
 * first value of the returned tuple represents the intermediary request state. The second value is
 * a promise that will resolve to the final value.
 */
var refreshRequest = function (rrd) { return function (run) {
    if (exports.isRefreshing(rrd))
        return function_1.tuple(rrd, Promise.resolve(rrd));
    var next = exports.refresh(rrd)(remote_data_ts_1.pending);
    var refreshNext = exports.refresh(next);
    return function_1.tuple(next, run().then(function_1.flow(remote_data_ts_1.fromEither, refreshNext)).catch(function_1.flow(remote_data_ts_1.failure, refreshNext)));
}; };
exports.refreshRequest = refreshRequest;
