import { RemoteData } from "@devexperts/remote-data-ts";
import { FunctionN } from "fp-ts/function";
import { TaskEither } from "fp-ts/TaskEither";
/**
 * The {@link RefreshableRemoteData} type represents a value that can be refreshed without losing
 * the existing state. It does so using a pair of {@link RemoteData} values where one always
 * represents the "current" value, while the other is used to keep track of network requests in the
 * background. Depending on the update strategy, the "current" value will eventually be replaced
 * by the "request" value.
 */
export declare type RefreshableRemoteData<E, A> = {
    current: RemoteData<E, A>;
    request: RemoteData<E, A>;
};
/**
 * Create a {@link RefreshableRemoteData} value from a {@link RemoteData} value
 */
export declare const fromRemoteData: <E, A>(current: RemoteData<E, A>) => RefreshableRemoteData<E, A>;
/**
 * If a request is pending, the {@link RefreshableRemoteData} is in a "refreshing" state and no
 * further network requests should be sent.
 */
export declare const isRefreshing: <E, A>({ request }: RefreshableRemoteData<E, A>) => boolean;
/**
 * Given a {@link RefreshableRemoteData}, the stale-while-revalidate refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - keep returning the current value until it is replaced by another success or failure value
 * - keep returning a failure value only as long as it is not being refreshed, in which case
 *   it will fall back to a pending value
 *
 * @see {@link staleIfError}
 */
export declare const staleWhileRevalidate: <E, A>(current: RemoteData<E, A>) => FunctionN<[RemoteData<E, A>], RefreshableRemoteData<E, A>>;
/**
 * Given a {@link RefreshableRemoteData}, the stale-if-error refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - return a failure only if no data has been previously fetched; newer failures replace older ones
 * - keep returning the cached success value indefinitely unless it is replaced by another success value
 *
 * @see {@link staleWhileRevalidate}
 */
export declare const staleIfError: <E, A>(current: RemoteData<E, A>) => FunctionN<[RemoteData<E, A>], RefreshableRemoteData<E, A>>;
/**
 * Build a function to update a {@link RefreshableRemoteData} with a new {@link RemoteData} value
 * using the given strategy.
 */
export declare const refreshWithStrategy: (strategy?: <E, A>(current: RemoteData<E, A>) => FunctionN<[RemoteData<E, A>], RefreshableRemoteData<E, A>>) => <E_1, A_1>(rrd: RefreshableRemoteData<E_1, A_1>) => FunctionN<[RemoteData<E_1, A_1>], RefreshableRemoteData<E_1, A_1>>;
/**
 * Transition a {@link RefreshableRemoteData} value to the next state using the
 * {@link staleWhileRevalidate} strategy.
 */
export declare const refreshSWR: <E, A>(rrd: RefreshableRemoteData<E, A>) => FunctionN<[RemoteData<E, A>], RefreshableRemoteData<E, A>>;
/**
 * Transition a {@link RefreshableRemoteData} value to the next state using the
 * {@link staleIfError} strategy.
 */
export declare const refreshSIE: <E, A>(rrd: RefreshableRemoteData<E, A>) => FunctionN<[RemoteData<E, A>], RefreshableRemoteData<E, A>>;
/**
 * Transition a {@link RefreshableRemoteData} value to the next state.
 * @see {@link refreshSWR}
 */
export declare const refresh: <E, A>(rrd: RefreshableRemoteData<E, A>) => FunctionN<[RemoteData<E, A>], RefreshableRemoteData<E, A>>;
/**
 * Transition a {@link RefreshableRemoteData} value to the next state using a network request. The
 * first value of the returned tuple represents the intermediary request state. The second value is
 * a promise that will resolve to the final value.
 */
export declare const refreshRequest: <E, A>(rrd: RefreshableRemoteData<E, A>) => (run: TaskEither<E, A>) => [RefreshableRemoteData<E, A>, Promise<RefreshableRemoteData<E, A>>];
