import { RemoteData } from '@devexperts/remote-data-ts';
/**
 * When refreshing a resource it can be desirable to do so silently in the background while keeping
 * the previously fetched value around until the new value is ready.
 *
 * The {@link RemoteDataRevalidate} type can express this by using a tuple of {@link RemoteData}
 * types, where
 *
 * 1. The first value represents the cached (stale) value that represents the latest loaded data
 * 2. The second value represents the background request that will refresh (revalidate) the cache
 */
export declare type RemoteDataRevalidate<E, A> = [
    RemoteData<E, A> /** Current state (cache) */,
    RemoteData<E, A> /** Next state (revalidation request) */
];
/**
 * Given a {@link RemoteDataRevalidate}, the stale-while-revalidate refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - keep returning a success value until it is replaced by another success or failure value
 * - keep returning a failure value only as long as it is not being refreshed, in which case
 *   it will fall back to a pending value
 *
 * @see {@link staleIfError}
 */
export declare function staleWhileRevalidate<E, A>([current, next]: RemoteDataRevalidate<E, A>): RemoteDataRevalidate<E, A>;
/**
 * Given a {@link RemoteDataRevalidate}, the stale-if-error refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - return a failure only if no data has been previously fetched; newer failures replace older ones
 * - keep returning the cached success value indefinitely unless it is replaced by another success value
 *
 * @see {@link staleWhileRevalidate}
 */
export declare function staleIfError<E, A>([current, next]: RemoteDataRevalidate<E, A>): RemoteDataRevalidate<E, A>;
