import { pipe } from 'fp-ts/function';
import { failure, fold, initial, isFailure, isInitial, isSuccess, success, } from '@devexperts/remote-data-ts';
/**
 * Given a {@link RemoteDataRevalidate}, the stale-while-revalidate refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - keep returning a success value until it is replaced by another success or failure value
 * - keep returning a failure value only as long as it is not being refreshed, in which case
 *   it will fall back to a pending value
 *
 * @see {@link staleIfError}
 */
export function staleWhileRevalidate([current, next]) {
    return pipe(next, fold(() => [current, next], () => [isInitial(current) || isFailure(current) ? next : current, next], (e) => [failure(e), initial], (x) => [success(x), initial]));
}
/**
 * Given a {@link RemoteDataRevalidate}, the stale-if-error refresh strategy will
 *
 * - return an initial or pending value as long as no data has been fetched
 * - return a failure only if no data has been previously fetched; newer failures replace older ones
 * - keep returning the cached success value indefinitely unless it is replaced by another success value
 *
 * @see {@link staleWhileRevalidate}
 */
export function staleIfError([current, next]) {
    return pipe(next, fold(() => [current, next], () => [isInitial(current) ? next : current, next], (e) => [isSuccess(current) ? current : failure(e), initial], (x) => [success(x), initial]));
}
